@@ -152,6 +152,8 @@ struct stats_memory {
 	unsigned long tlskb	__attribute__ ((aligned (8)));
 	unsigned long caskb	__attribute__ ((aligned (8)));
 	unsigned long comkb	__attribute__ ((aligned (8)));
+	unsigned long activekb	__attribute__ ((aligned (8)));
+	unsigned long inactkb	__attribute__ ((aligned (8)));
 };
 
 #define STATS_MEMORY_SIZE	(sizeof(struct stats_memory))
@@ -169,6 +171,7 @@ struct stats_ktables {
 /* Structure for queue and load statistics */
 struct stats_queue {
 	unsigned long nr_running	__attribute__ ((aligned (8)));
+	unsigned long procs_blocked	__attribute__ ((aligned (8)));
 	unsigned int  load_avg_1	__attribute__ ((aligned (8)));
 	unsigned int  load_avg_5	__attribute__ ((packed));
 	unsigned int  load_avg_15	__attribute__ ((packed));
@@ -486,6 +489,7 @@ struct stats_pwr_cpufreq {
  */
 struct stats_pwr_fan {
 	double  rpm				__attribute__ ((aligned (8)));
+	double  rpm_min				__attribute__ ((aligned (8)));
 	char    device[MAX_SENSORS_DEV_LEN]	__attribute__ ((aligned (8)));
 };
 
@@ -496,11 +500,45 @@ struct stats_pwr_fan {
  */
 struct stats_pwr_temp {
 	double  temp				__attribute__ ((aligned (8)));
+	double  temp_min			__attribute__ ((aligned (8)));
+	double  temp_max			__attribute__ ((aligned (8)));
 	char    device[MAX_SENSORS_DEV_LEN]	__attribute__ ((aligned (8)));
 };
 
 #define STATS_PWR_TEMP_SIZE    (sizeof(struct stats_pwr_temp))
 
+/*
+ * Structure for voltage inputs statistics.
+ */
+struct stats_pwr_in {
+	double  in				__attribute__ ((aligned (8)));
+	double  in_min				__attribute__ ((aligned (8)));
+	double  in_max				__attribute__ ((aligned (8)));
+	char    device[MAX_SENSORS_DEV_LEN]	__attribute__ ((aligned (8)));
+};
+
+#define STATS_PWR_IN_SIZE	(sizeof(struct stats_pwr_in))
+
+/* Structure for hugepages statistics */
+struct stats_huge {
+	unsigned long frhkb			__attribute__ ((aligned (8)));
+	unsigned long tlhkb			__attribute__ ((aligned (8)));
+};
+
+#define STATS_HUGE_SIZE	(sizeof(struct stats_memory))
+
+/*
+ * Structure for weighted CPU frequency statistics.
+ * In activity buffer: First structure is for global CPU utilisation ("all").
+ * Following structures are for each individual CPU (0, 1, etc.)
+ */
+struct stats_pwr_wghfreq {
+	unsigned long long 	time_in_state	__attribute__ ((aligned (16)));
+	unsigned long 		freq		__attribute__ ((aligned (16)));
+};
+
+#define STATS_PWR_WGHFREQ_SIZE	(sizeof(struct stats_pwr_wghfreq))
+
 /*
  ***************************************************************************
  * Prototypes for functions used to read system statistics
@@ -574,6 +612,12 @@ extern void
 	read_fan(struct stats_pwr_fan *, int);
 extern void
 	read_temp(struct stats_pwr_temp *, int);
+extern void
+	read_in(struct stats_pwr_in *, int);
+extern void
+	read_meminfo_huge(struct stats_huge *);
+extern void
+	read_time_in_state(struct stats_pwr_wghfreq *, int, int);
 
 /*
  ***************************************************************************
@@ -599,5 +643,9 @@ extern int
 	get_fan_nr(void);
 extern int
 	get_temp_nr(void);
+extern int
+	get_in_nr(void);
+extern int
+	get_freq_nr(void);
 
 #endif /* _RD_STATS_H */
