@@ -1,6 +1,6 @@
 /*
  * sar: report system activity
- * (C) 1999-2009 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1999-2010 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -50,6 +50,8 @@ long interval = -1, count = 0;
 int dis = TRUE;
 
 unsigned int flags = 0;
+unsigned int dm_major;	/* Device-mapper major number */
+
 char timestamp[2][TIMESTAMP_LEN];
 
 unsigned long avg_count = 0;
@@ -102,10 +104,10 @@ void usage(char *progname)
 
 	print_usage_title(progname);
 	fprintf(stderr, _("Options are:\n"
-			  "[ -A ] [ -b ] [ -B ] [ -C ] [ -d ] [ -h ] [ -m ] [ -p ] [ -q ] [ -r ]\n"
+			  "[ -A ] [ -b ] [ -B ] [ -C ] [ -d ] [ -h ] [ -H ] [ -p ] [ -q ] [ -r ]\n"
 			  "[ -R ] [ -S ] [ -t ] [ -u [ ALL ] ] [ -v ] [ -V ] [ -w ] [ -W ] [ -y ]\n"
 			  "[ -I { <int> [,...] | SUM | ALL | XALL } ] [ -P { <cpu> [,...] | ALL } ]\n"
-			  "[ -n { <keyword> [,...] | ALL } ]\n"
+			  "[ -m { <keyword> [,...] | ALL } ] [ -n { <keyword> [,...] | ALL } ]\n"
 			  "[ -o [ <filename> ] | -f [ <filename> ] ]\n"
 			  "[ -i <interval> ] [ -s [ <hh:mm:ss> ] ] [ -e [ <hh:mm:ss> ] ]\n"));
 	exit(1);
@@ -127,9 +129,17 @@ void display_help(char *progname)
 	fprintf(stderr, _("\t-b\tI/O and transfer rate statistics\n"));
 	fprintf(stderr, _("\t-B\tPaging statistics\n"));
 	fprintf(stderr, _("\t-d\tBlock device statistics\n"));
+	fprintf(stderr, _("\t-H\tHugepages utilization statistics\n"));
 	fprintf(stderr, _("\t-I { <int> | SUM | ALL | XALL }\n"
 			  "\t\tInterrupts statistics\n"));
-	fprintf(stderr, _("\t-m\tPower management statistics\n"));
+	fprintf(stderr, _("\t-m { <keyword> [,...] | ALL }\n"
+			  "\t\tPower management statistics\n"
+			  "\t\tKeywords are:\n"
+			  "\t\tCPU\tCPU instantaneous clock frequency\n"
+			  "\t\tFAN\tFans speed\n"
+			  "\t\tFREQ\tCPU average clock frequency\n"
+			  "\t\tIN\tVoltage inputs\n"
+			  "\t\tTEMP\tDevices temperature\n"));
 	fprintf(stderr, _("\t-n { <keyword> [,...] | ALL }\n"
 			  "\t\tNetwork statistics\n"
 			  "\t\tKeywords are:\n"
@@ -211,7 +221,7 @@ void print_read_error(void)
  ***************************************************************************
  * Check that every selected activity actually belongs to the sequence list.
  * If not, then the activity should be unselected since it will not be sent
- * by sadc. An activity can be unsent if its number of items is null.
+ * by sadc. An activity can be not sent if its number of items is null.
  *
  * IN:
  * @act_nr	Size of sequence list.
@@ -237,17 +247,20 @@ void reverse_check_act(unsigned int act_nr)
 
 /*
  ***************************************************************************
- * Fill the rectime structure with current record's time, based on current
- * record's time data saved in file.
+ * Fill the (struct tm) rectime structure with current record's time,
+ * based on current record's time data saved in file.
  * The resulting timestamp is expressed in the locale of the file creator
  * or in the user's own locale depending on whether option -t has been used
  * or not.
  *
  * IN:
  * @curr	Index in array for current sample statistics.
+ *
+ * RETURNS:
+ * 1 if an error was detected, or 0 otherwise.
  ***************************************************************************
 */
-void sar_get_record_timestamp_struct(int curr)
+int sar_get_record_timestamp_struct(int curr)
 {
 	struct tm *ltm;
 
@@ -259,9 +272,17 @@ void sar_get_record_timestamp_struct(int curr)
 		rectime.tm_sec  = record_hdr[curr].second;
 	}
 	else {
-		ltm = localtime((const time_t *) &record_hdr[curr].ust_time);
+		if ((ltm = localtime((const time_t *) &record_hdr[curr].ust_time)) == NULL)
+			/*
+			 * An error was detected.
+			 * The rectime structure has NOT been updated.
+			 */
+			return 1;
+		
 		rectime = *ltm;
 	}
+	
+	return 0;
 }
 
 /*
@@ -310,15 +331,22 @@ int check_line_hdr(void)
  *
  * OUT:
  * @cur_time	Timestamp string.
+ *
+ * RETURNS:
+ * 1 if an error was detected, or 0 otherwise.
  ***************************************************************************
 */
-void set_record_timestamp_string(int curr, char *cur_time, int len)
+int set_record_timestamp_string(int curr, char *cur_time, int len)
 {
 	/* Fill timestamp structure */
-	sar_get_record_timestamp_struct(curr);
+	if (sar_get_record_timestamp_struct(curr))
+		/* Error detected */
+		return 1;
 
 	/* Set cur_time date value */
 	strftime(cur_time, len, "%X", &rectime);
+
+	return 0;
 }
 
 /*
@@ -400,7 +428,7 @@ void write_stats_avg(int curr, int read_from_file, unsigned int act_id)
  * @cnt			Number of remaining lines to display.
  *
  * RETURNS:
- * 1 if stats have been successfully displayed.
+ * 1 if stats have been successfully displayed, and 0 otherwise.
  ***************************************************************************
  */
 int write_stats(int curr, int read_from_file, long *cnt, int use_tm_start,
@@ -423,9 +451,11 @@ int write_stats(int curr, int read_from_file, long *cnt, int use_tm_start,
 	}
 
 	/* Set previous timestamp */
-	set_record_timestamp_string(!curr, timestamp[!curr], 16);
+	if (set_record_timestamp_string(!curr, timestamp[!curr], 16))
+		return 0;
 	/* Set current timestamp */
-	set_record_timestamp_string(curr,  timestamp[curr],  16);
+	if (set_record_timestamp_string(curr,  timestamp[curr],  16))
+		return 0;
 
 	/* Check if we are beginning a new day */
 	if (use_tm_start && record_hdr[!curr].ust_time &&
@@ -502,8 +532,9 @@ void write_stats_startup(int curr)
 	record_hdr[!curr].ust_time    = record_hdr[curr].ust_time;
 
 	for (i = 0; i < NR_ACT; i++) {
-		if (IS_SELECTED(act[i]->options) && (act[i]->nr > 0))
-			memset(act[i]->buf[!curr], 0, act[i]->msize * act[i]->nr);
+		if (IS_SELECTED(act[i]->options) && (act[i]->nr > 0)) {
+			memset(act[i]->buf[!curr], 0, act[i]->msize * act[i]->nr * act[i]->nr2);
+		}
 	}
 	
 	flags |= S_F_SINCE_BOOT;
@@ -562,7 +593,7 @@ int sa_read(void *buffer, int size)
  * @ifd			Input file descriptor.
  *
  * RETURNS:
- * 1 if the record has been successfully displayed.
+ * 1 if the record has been successfully displayed, and 0 otherwise.
  ***************************************************************************
  */
 int sar_print_special(int curr, int use_tm_start, int use_tm_end, int rtype, int ifd)
@@ -570,7 +601,8 @@ int sar_print_special(int curr, int use_tm_start, int use_tm_end, int rtype, int
 	char cur_time[26];
 	int dp = 1;
 
-	set_record_timestamp_string(curr, cur_time, 26);
+	if (set_record_timestamp_string(curr, cur_time, 26))
+		return 0;
 
 	/* The record must be in the interval specified by -s/-e options */
 	if ((use_tm_start && (datecmp(&rectime, &tm_start) < 0)) ||
@@ -611,7 +643,7 @@ int sar_print_special(int curr, int use_tm_start, int use_tm_end, int rtype, int
 void read_sadc_stat_bunch(int curr)
 {
 	int i, p;
-	
+
 	/* Read record header (type is always R_STATS since it is read from sadc) */
 	if (sa_read(&record_hdr[curr], RECORD_HEADER_SIZE)) {
 		print_read_error();
@@ -624,8 +656,8 @@ void read_sadc_stat_bunch(int curr)
 		if ((p = get_activity_position(act, id_seq[i])) < 0) {
 			PANIC(1);
 		}
-		
-		if (sa_read(act[p]->buf[curr], act[p]->fsize * act[p]->nr)) {
+
+		if (sa_read(act[p]->buf[curr], act[p]->fsize * act[p]->nr * act[p]->nr2)) {
 			print_read_error();
 		}
 	}
@@ -794,13 +826,18 @@ void read_header_data(void)
 
 		p = get_activity_position(act, file_act.id);
 
-		if ((p < 0) || (act[p]->fsize != file_act.size) || !file_act.nr) {
+		if ((p < 0) || (act[p]->fsize != file_act.size)
+			    || !file_act.nr
+			    || !file_act.nr2
+			    || (act[p]->magic != file_act.magic)) {
+			/* Remember that we are reading data from sadc and not from a file... */
 			fprintf(stderr, _("Inconsistent input data\n"));
 			exit(3);
 		}
 
 		id_seq[i]   = file_act.id;	/* We necessarily have "i < NR_ACT" */
 		act[p]->nr  = file_act.nr;
+		act[p]->nr2 = file_act.nr2;
 	}
 
 	while (i < NR_ACT) {
@@ -865,7 +902,12 @@ void read_stats_from_file(char from_file[])
 				 */
 				read_file_stat_bunch(act, 0, ifd, file_hdr.sa_nr_act,
 						     file_actlst);
-				sar_get_record_timestamp_struct(0);
+				if (sar_get_record_timestamp_struct(0))
+					/*
+					 * An error was detected.
+					 * The timestamp hasn't been updated.
+					 */
+					continue;
 			}
 		}
 		while ((rtype == R_RESTART) || (rtype == R_COMMENT) ||
@@ -883,7 +925,10 @@ void read_stats_from_file(char from_file[])
 			exit(2);
 		}
 
-		/* Read and write stats located between two possible Linux restarts */
+		/*
+		 * Read and write stats located between two possible Linux restarts.
+		 * Activities that should be displayed are saved in id_seq[] array.
+		 */
 		for (i = 0; i < NR_ACT; i++) {
 
 			if (!id_seq[i])
@@ -1040,7 +1085,7 @@ void read_stats(void)
  */
 int main(int argc, char **argv)
 {
-	int opt = 1, args_idx = 2;
+	int i, opt = 1, args_idx = 2;
 	int fd[2];
 	char from_file[MAX_FILE_LEN], to_file[MAX_FILE_LEN];
 	char ltemp[20];
@@ -1141,6 +1186,18 @@ int main(int argc, char **argv)
 			flags |= S_F_INTERVAL_SET;
 		}
 
+		else if (!strcmp(argv[opt], "-m")) {
+			if (argv[++opt]) {
+				/* Parse option -m */
+				if (parse_sar_m_opt(argv, &opt, act)) {
+					usage(argv[0]);
+				}
+			}
+			else {
+				usage(argv[0]);
+			}
+		}
+
 		else if (!strcmp(argv[opt], "-n")) {
 			if (argv[++opt]) {
 				/* Parse option -n */
@@ -1218,6 +1275,10 @@ int main(int argc, char **argv)
 		usage(argv[0]);
 	}
 
+	if (USE_PRETTY_OPTION(flags)) {
+		dm_major = get_devmap_major();
+	}
+	
 	if (!count) {
 		/*
 		 * count parameter not set: Display all the contents of the file
@@ -1293,13 +1354,35 @@ int main(int argc, char **argv)
 
 		/* Flags to be passed to sadc */
 		salloc(args_idx++, "-z");
-		salloc(args_idx++, "-S");
-		salloc(args_idx++, K_ALL);
-
-		/* Outfile arg */
+		
+		/* Writing data to a file (option -o) */
 		if (to_file[0]) {
+			/* Collect all possible activities (option -S ALL for sadc) */
+			salloc(args_idx++, "-S");
+			salloc(args_idx++, K_ALL);
+			/* Outfile arg */
 			salloc(args_idx++, to_file);
 		}
+		else {
+			/*
+			 * If option -o hasn't been used, then tell sadc
+			 * to collect only activities that will be displayed.
+			 */
+			int act_id = 0;
+			
+			for (i = 0; i < NR_ACT; i++) {
+				if (IS_SELECTED(act[i]->options)) {
+					act_id |= act[i]->group;
+				}
+			}
+			if (act_id) {
+				act_id <<= 8;
+				snprintf(ltemp, 19, "%d", act_id);
+				ltemp[19] = '\0';
+				salloc(args_idx++, "-S");
+				salloc(args_idx++, ltemp);
+			}
+		}
 
 		/* Last arg is NULL */
 		args[args_idx] = NULL;
